<template>
    <div class="container">
      <h1 class="fs-3 fl-4">Task Management</h1>

      <div class="mt-8 actions fl-5">
        <div v-if="currentTask.status == 'NOTCREATED'" class="mb-8" v-html="contents ['subtext:NOTCREATED']"></div>
        <div v-else-if="currentTask.status == 'COMPLETED'" class="mb-8" v-html="contents ['subtext:COMPLETED']"></div>
        <div v-else-if="currentTask.status == 'TUNING'" class="mb-8" v-html="contents ['subtext:TUNING']"></div>
        <div v-else-if="currentTask.status == 'TUNED'" class="mb-8" v-html="contents ['subtext:TUNED']"></div>
        <div v-else class="mb-8" v-html="contents ['subtext:INPROGRESS']"></div>
        <div class="actions">
          <router-link @click="resetCurrentTask ()" v-if="currentTask.status == 'NOTCREATED'" class="mr-2 btn btn-primary" to="/tasks/setup-image">Create New Task</router-link>
          <span v-else-if="currentTask.status == 'TUNING'">
            <router-link class="mr-2 btn btn-primary" to="/tasks/settings" v-if="currentTask.queued > 0">Tune/Test Task Configuration</router-link>
          </span>
          <span v-else-if="currentTask.status == 'TUNED'">
            <router-link class="mr-2 btn btn-primary" to="/tasks/add-images">Tuning Complete, Submit Images</router-link>
            <router-link class="mr-2 btn btn-outline-secondary" to="/tasks/settings">View Task Configuration</router-link>
          </span>
          <span v-else-if="currentTask.status == 'INPROGRESS'">
            <router-link class="mr-2 btn btn-primary" to="/tasks/analysis">Resume Active Task</router-link>
            <router-link class="mr-2 btn btn-outline-secondary" to="/tasks/settings">View Task Configuration</router-link>
          </span>
          <span v-else>
            <router-link class="mr-2 btn btn-success" to="/tasks/analysis">Review Active Task Result</router-link>
            <router-link class="mr-2 btn btn-outline-secondary" to="/tasks/settings">View Task Configuration</router-link>
            <button class="ml-4 btn btn-primary" @click="createNewTask ()">Create New Task</button>
          </span>
          <span class="ml-4">
            <button class="mr-4 btn btn-warning" @click="deleteTask ()" v-if="currentTask.status == 'INPROGRESS'">Cancel Active Task</button>
            <button class="mr-4 btn btn-danger" @click="deleteTask ()" v-else-if="currentTask.status == 'TUNING' || currentTask.status == 'TUNED'">Delete Active Task</button>
          </span>
        </div>
      </div>

      <div class="mt-4 fl-4">
        <scrollable ref="scroller" :items="tasks" height="370px" @scrollend="(currentPage) => pagenate (currentPage)">
          <table class="table table-sm table-dark table-hover table-striped">
            <thead>
              <tr>
                <th scope="col">Task Name</th>
                <th scope="col">Status<br>Task ID</th>
                <th scope="col" class="text-end pr-2">Progress<br>Credit Usage</th>
                <th scope="col" class="text-end pr-2">Created<br>Updated</th>
                <th scope="col">Results</th>
                <th scope="col">Manage</th>
              </tr>
            </thead>

            <tbody v-if="tasks.length == 0">
              <tr>
                <td colspan="20" class="text-center py-8" v-html="contents ['help:no-task']"></td>
              </tr>
            </tbody>

            <tbody>
              <tr v-for="task in tasks">
                <td class="align-middle py-2" style="min-width: 120px;">
                  <div>
                    <span v-if='task.id == currentTask.id' class="badge bg-yellow-lighten-3 mr-2">Active</span>
                    <router-link :to="`/tasks/${task.id}`" class="fw-bolder" style="text-decoration: none;">{{ task.name }}</router-link>
                    <a href="#" class="btn btn-sm text-grey py-0" v-if="task.status != 'TUNING' && task.task_type != 9" @click="taskName.showTaskNameDialog (task)"><i class="bi bi-pencil-square"></i></a>
                  </div>
                  <div class="text-grey-lighten-1 fs-14px pt-1">{{ _TASK_TYPES [task.task_type].name }}</div>
                </td>
                <td class="align-middle" style="min-width: 60px;">
                  <div><span class="badge px-0 fs-14px" :title="_TASK_TYPES [task.task_type].name" :class="'text-' + statusNames [task.status].color">{{ statusNames [task.status].name }}</span></div>
                  <div class="fs-14px pt-1 text-grey-lighten-1">ID-{{ task.id }}</div>
                </td>

                <td class="text-end text-nowrap align-middle">
                  <div>
                    <div v-if='task.status=="TUNING"'>- / {{ task.queued.format () }} <span class="text-grey">(-%)</span></div>
                    <div v-else-if="task.task_type == 9">{{ task.completed.format () }} / - <span class="text-grey">(-%)</span></div>
                    <div v-else>{{ task.completed.format () }} / {{ task.queued.format () }} <span class="text-grey">({{ task.queued > 0 ? Math.round (task.completed / task.queued * 100):0 }}%)</span></div>
                  </div>
                  <div class="fs-14px pt-1">
                    <b class="text-red-lighten-3"><span>{{ task.credits_used.format () }}</span></b> <span class="text-grey ml-1">Credit(s)</span>
                  </div>
                </td>
                <td class="text-end text-nowrap align-middle fs-14px">
                  <div class="pr-2 mb-1">{{ task.creation_date.dateformat ("%b %d, %Y") }}</div>
                  <div class="pr-2">{{ task.last_updated.dateformat ("%b %d, %Y") }}</div>
                </td>
                <td style="width: 75px;" class="align-middle">
                  <button type="button" class="btn btn-sm mr-2 btn-info" :class="{disabled: task.task_type == 9 || (task.status != 'COMPLETED' && task.status != 'CANCELED')}" @click="downloadResult (task.id)">Download</button>
                </td>
                <td style="width: 50px;" class="align-middle">
                  <div v-if="task.task_type == 9">
                    <button v-if="task.status == 'INPROGRESS'" type="button" class="btn btn-sm mr-2 btn-danger" @click="cantDeleteTask (task)">Delete</button>
                    <button v-else type="button" class="btn btn-sm mr-2 btn-danger" @click="deleteTask (task)">Delete</button>
                  </div>
                  <div v-else>
                    <button type="button" class="btn btn-sm mr-2 btn-warning" @click="deleteTask (task)" v-if="task.status == 'INPROGRESS'">Cancel</button>
                    <button type="button" class="btn btn-sm mr-2 btn-danger" @click="deleteTask (task)" v-else>Delete</button>
                  </div>
                </td>

              </tr>
            </tbody>
          </table>
        </scrollable>

      </div>




      <task-name ref="taskName" mode="rename" :current-task="currentTask"></task-name>

      <div class="modal" tabindex="-1" id="progressModal" data-bs-backdrop="static">
        <div class="modal-dialog">
          <div class="modal-content">
            <div class="modal-body">
              <div class="col-12">
                <div class="spinner-border spinner-border-sm text-warning mx-1" role="status">
                  <span class="visually-hidden">Loading...</span>
                </div>
                Downloading JSON Result... (<span class="fw-bold text-success">{{ downloadedReadable }}</span>)
              </div>
            </div>
          </div>
        </div>
      </div>


    </div>

</template>


<script setup>
import TaskName from "./components/task-name.vue"
import Scrollable from "/atila-vue/components/scrollable.vue"

const taskName = ref (null)
const scroller = ref (null)
const emitter = inject ('emitter')

const ITEM_PER_PAGE = 20
const tasks = ref ([])
const router = useRouter ()
const { state } = useStore ()
const currentTask = ref ({status: 'NOTCREATED', id: null})
const downloaded = ref (0)
const contents = ref ({})
const hasApiTask = ref (false)
const statusNames = TASK_STATUS_NAME
const _TASK_TYPES = TASK_TYPES

const downloadedReadable = computed (() => {
  const val = downloaded.value
  if (val > 1000000) return `${(val / 1000000).toFixed (2)} MB`
  if (val > 1000) return `${parseInt (val / 1000)} KB`
  return `${val} B`
})

function cantDeleteTask (task) {
  bootstrap.modal ({
    title: `Delete API Project Task`,
    closeButtonText: "Cancel",
    text: `To delete task <b>${task.name}</b>, you must first delete the associated API key.`,
    icon: "danger",
    buttons: [{text: `View API Keys`, class_: "btn-primary", to: '/credit/api-keys'}]
  })
}

async function resetCurrentTask () {
  currentTask.value.status = 'NOTCREATED'
  currentTask.value.id = null
  state.$ls.set (state.$g.taskKey, currentTask.value)
  await state.$idb.reinitialize ()
}

function createNewTask () {
  function _proceed () {
    resetCurrentTask ()
    router.push ('/tasks/setup-image')
  }
  bootstrap.modal ({
    title: "Creating a New Task",
    text: contents.value ["modal:overwrite"],
    closeButtonText: 'Cancel',
    buttons: [{text: `Understood, Create New Task`, class_: "btn-primary", run: _proceed}]
  })
}

async function downloadResult (task_id) {
  const progressModal = new bootstrap.Modal(document.getElementById ('progressModal'))
  progressModal.show ()
  downloaded.value = 0
  await backend.download ('DOWNLOAD_TASK_RESULT',
    {task_id: task_id},
    {onDownloadProgress: progressEvent => {
      downloaded.value = progressEvent.loaded
    }}
  )
  progressModal.hide ()
}

function deleteTask (task) {
  async function _deleteTask () {
    emitter.emit ('task-deleted')
    const resp = await backend.delete ('GET_DELETE_TASK', {task_id: task.id})
    if (action.isCurrentTask) {
      resetCurrentTask ()
    } else {
      const task_index = tasks.value.findIndex ( (item) => item.id == task.id)
    }
    bootstrap.toast ({title: "Success", text: `Task has been successfully ${action.result}.`})
    listTasks ()
  }

  const action = {isCurrentTask: false}
  if (!task) {
    const task_index = tasks.value.findIndex ( (item) => item.id == currentTask.value.id)
    task = tasks.value [task_index]
  }
  if (task.id == currentTask.value.id) {
    action.isCurrentTask = true
  }
  if (task.status == 'INPROGRESS') {
    action.name = 'Cancel'
    action.result = 'canceled'
    action.text = contents.value ['modal:cancel']
  } else {
    action.name = 'Delete'
    action.result = 'deleted'
    action.text = contents.value ['modal:delete']
  }
  action.text += `<p>Are you absolutely sure to <b>${action.name.toLocaleLowerCase ()} this task</b>?</p>`

  bootstrap.modal ({
    title: `${action.name} Task`,
    text: action.text,
    closeButtonText: 'Cancel',
    textConfirm: task.name,
    icon: "danger",
    buttons: [{text: `${action.name} Task`, class_: "btn-danger", run: _deleteTask}]
  })
}

async function pagenate (pageId = 0) {
  const resp = await backend.get ('GET_TASK_LIST', {offset: pageId * ITEM_PER_PAGE, fetch: ITEM_PER_PAGE})
  for (const task of resp.data.result) {
    if (['TUNING', 'TUNED', 'DELETED'].indexOf (task.status) != -1) {
      continue
    }
    if (task.task_type == 9) {
      hasApiTask.value = true
    }
    if (task.status == 'INPROGRESS') {
      if (task.id != currentTask.value.id) {
        tasks.value.push (task)
      } else {
        tasks.value.splice(0, 0, task)
      }
    }
    else {
      tasks.value.push (task)
    }

  }
  scroller.value.updated (resp.data.result.length, pageId)
}

async function listTasks () {
  function compare (a, b) {
    if (a.id < b.id) return 1
    if (a.id > b.id) return -1
    return 0
  }

  tasks.value.splice (0, tasks.value.length)
  if (currentTask.value.status == 'TUNING' || currentTask.value.status == 'TUNED') {
    currentTask.value.credits_used = 0
    currentTask.value.completed = 0
    tasks.value.push (currentTask.value)
  }
  pagenate (0)
}

emitter.on ('task-finished', () => {
  if (!tasks.value.length) {
    return
  }
  currentTask.value = state.$ls.get (state.$g.taskKey)
  tasks.value [0].status = 'COMPLETED'
  tasks.value [0].name = currentTask.value.name
})

emitter.on ('item-analyzed', (successCount) => {
  if (!tasks.value.length) {
    return
  }
  tasks.value [0].completed = successCount
  tasks.value [0].credits_used = successCount * currentTask.value.setting.options.road_length_m
  tasks.value [0].last_updated = new Date ().toDateString ()
})

onActivated (async () => {
  frontend.set_title ()
  await frontend.wait_for_define ('store.state.$g.taskKey')
  const savedTask = state.$ls.get (state.$g.taskKey)
  if (savedTask != null) {
    currentTask.value = savedTask
    if (!currentTask.value.task_type) {
      currentTask.value.task_type = currentTask.value.setting.options.focal_length_mm ? 1:0
      state.$ls.set (state.$g.taskKey, currentTask.value)
    }
  }
  contents.value = await frontend.fetch_contents ({taskName: currentTask.value.name})
  await listTasks ()
})
</script>

<script>
export default {
  beforeRouteEnter (to, from, next) {
    frontend.permission.required (['user'], {path: '/members/signin'}, {to, from, next})
  }
}
</script>
